@@ -19,20 +19,27 @@
 #include "Encoding.h"
 #include "ErrorList.h"
 #include "Format.h"
+#include "InsetList.h"
 #include "Language.h"
 #include "LaTeX.h"
+#include "LyXRC.h"
 #include "Mover.h"
+#include "ParagraphList.h"
+#include "Session.h"
 
 #include "frontends/alert.h"
 
+#include "insets/InsetInclude.h"
+
 #include "support/debug.h"
 #include "support/FileNameList.h"
 #include "support/filetools.h"
 #include "support/gettext.h"
+#include "support/lassert.h"
 #include "support/lstrings.h"
 #include "support/os.h"
 #include "support/Package.h"
-#include "support/Path.h"
+#include "support/PathChanger.h"
 #include "support/Systemcall.h"
 
 using namespace std;
@@ -50,7 +57,9 @@ string const token_base("$$b");
 string const token_to("$$o");
 string const token_path("$$p");
 string const token_orig_path("$$r");
+string const token_orig_from("$$f");
 string const token_encoding("$$e");
+string const token_latex_encoding("$$E");
 
 
 string const add_options(string const & command, string const & options)
@@ -83,69 +92,60 @@ public:
 	ConverterEqual(string const & from, string const & to)
 		: from_(from), to_(to) {}
 	bool operator()(Converter const & c) const {
-		return c.from == from_ && c.to == to_;
+		return c.from() == from_ && c.to() == to_;
 	}
 private:
 	string const from_;
 	string const to_;
 };
 
-} // namespace anon
+} // namespace
 
 
 Converter::Converter(string const & f, string const & t,
 		     string const & c, string const & l)
-	: from(f), to(t), command(c), flags(l),
-	  From(0), To(0), latex(false), xml(false),
-	  need_aux(false)
+	: from_(f), to_(t), command_(c), flags_(l),
+	  From_(0), To_(0), latex_(false), xml_(false),
+	  need_aux_(false), nice_(false), need_auth_(false)
 {}
 
 
 void Converter::readFlags()
 {
-	string flag_list(flags);
+	string flag_list(flags_);
 	while (!flag_list.empty()) {
 		string flag_name, flag_value;
 		flag_list = split(flag_list, flag_value, ',');
 		flag_value = split(flag_value, flag_name, '=');
 		if (flag_name == "latex") {
-			latex = true;
-			latex_flavor = flag_value.empty() ?
+			latex_ = true;
+			latex_flavor_ = flag_value.empty() ?
 				"latex" : flag_value;
 		} else if (flag_name == "xml")
-			xml = true;
+			xml_ = true;
 		else if (flag_name == "needaux")
-			need_aux = true;
+			need_aux_ = true;
 		else if (flag_name == "resultdir")
-			result_dir = (flag_value.empty())
+			result_dir_ = (flag_value.empty())
 				? token_base : flag_value;
 		else if (flag_name == "resultfile")
-			result_file = flag_value;
+			result_file_ = flag_value;
 		else if (flag_name == "parselog")
-			parselog = flag_value;
+			parselog_ = flag_value;
+		else if (flag_name == "nice")
+			nice_ = true;
+		else if (flag_name == "needauth")
+			need_auth_ = true;
+		else if (flag_name == "hyperref-driver")
+			href_driver_ = flag_value;
 	}
-	if (!result_dir.empty() && result_file.empty())
-		result_file = "index." + formats.extension(to);
+	if (!result_dir_.empty() && result_file_.empty())
+		result_file_ = "index." + theFormats().extension(to_);
 	//if (!contains(command, token_from))
 	//	latex = true;
 }
 
 
-bool operator<(Converter const & a, Converter const & b)
-{
-	// use the compare_ascii_no_case instead of compare_no_case,
-	// because in turkish, 'i' is not the lowercase version of 'I',
-	// and thus turkish locale breaks parsing of tags.
-	int const i = compare_ascii_no_case(a.From->prettyname(),
-					    b.From->prettyname());
-	if (i == 0)
-		return compare_ascii_no_case(a.To->prettyname(),
-					     b.To->prettyname()) < 0;
-	else
-		return i < 0;
-}
-
-
 Converter const * Converters::getConverter(string const & from,
 					    string const & to) const
 {
@@ -174,8 +174,8 @@ int Converters::getNumber(string const & from, string const & to) const
 void Converters::add(string const & from, string const & to,
 		     string const & command, string const & flags)
 {
-	formats.add(from);
-	formats.add(to);
+	theFormats().add(from);
+	theFormats().add(to);
 	ConverterList::iterator it = find_if(converterlist_.begin(),
 					     converterlist_.end(),
 					     ConverterEqual(from , to));
@@ -183,27 +183,36 @@ void Converters::add(string const & from, string const & to,
 	Converter converter(from, to, command, flags);
 	if (it != converterlist_.end() && !flags.empty() && flags[0] == '*') {
 		converter = *it;
-		converter.command = command;
-		converter.flags = flags;
+		converter.setCommand(command);
+		converter.setFlags(flags);
 	}
 	converter.readFlags();
 
-	// If we have both latex & pdflatex, we set latex_command to latex.
 	// The latex_command is used to update the .aux file when running
 	// a converter that uses it.
-	if (converter.latex
-	    && (latex_command_.empty() || converter.latex_flavor == "latex"))
-		latex_command_ = subst(command, token_from, "");
-	// Similarly, set xelatex_command to xelatex.
-	if (converter.latex
-	    && (xelatex_command_.empty() || converter.latex_flavor == "xelatex"))
-		xelatex_command_ = subst(command, token_from, "");
+	if (converter.latex()) {
+		if (latex_command_.empty() ||
+		    converter.latex_flavor() == "latex")
+			latex_command_ = subst(command, token_from, "");
+		if (dvilualatex_command_.empty() ||
+		    converter.latex_flavor() == "dvilualatex")
+			dvilualatex_command_ = subst(command, token_from, "");
+		if (lualatex_command_.empty() ||
+		    converter.latex_flavor() == "lualatex")
+			lualatex_command_ = subst(command, token_from, "");
+		if (pdflatex_command_.empty() ||
+		    converter.latex_flavor() == "pdflatex")
+			pdflatex_command_ = subst(command, token_from, "");
+		if (xelatex_command_.empty() ||
+		    converter.latex_flavor() == "xelatex")
+			xelatex_command_ = subst(command, token_from, "");
+	}
 
 	if (it == converterlist_.end()) {
 		converterlist_.push_back(converter);
 	} else {
-		converter.From = it->From;
-		converter.To = it->To;
+		converter.setFrom(it->From());
+		converter.setTo(it->To());
 		*it = converter;
 	}
 }
@@ -232,8 +241,8 @@ void Converters::update(Formats const & formats)
 	ConverterList::iterator it = converterlist_.begin();
 	ConverterList::iterator end = converterlist_.end();
 	for (; it != end; ++it) {
-		it->From = formats.getFormat(it->from);
-		it->To = formats.getFormat(it->to);
+		it->setFrom(formats.getFormat(it->from()));
+		it->setTo(formats.getFormat(it->to()));
 	}
 }
 
@@ -244,36 +253,148 @@ void Converters::updateLast(Formats const & formats)
 {
 	if (converterlist_.begin() != converterlist_.end()) {
 		ConverterList::iterator it = converterlist_.end() - 1;
-		it->From = formats.getFormat(it->from);
-		it->To = formats.getFormat(it->to);
+		it->setFrom(formats.getFormat(it->from()));
+		it->setTo(formats.getFormat(it->to()));
 	}
 }
 
 
-void Converters::sort()
-{
-	std::sort(converterlist_.begin(), converterlist_.end());
-}
-
-
-OutputParams::FLAVOR Converters::getFlavor(Graph::EdgePath const & path)
+OutputParams::FLAVOR Converters::getFlavor(Graph::EdgePath const & path,
+					   Buffer const * buffer)
 {
 	for (Graph::EdgePath::const_iterator cit = path.begin();
 	     cit != path.end(); ++cit) {
 		Converter const & conv = converterlist_[*cit];
-		if (conv.latex)
-			if (conv.latex_flavor == "xelatex")
+		if (conv.latex()) {
+			if (conv.latex_flavor() == "latex")
+				return OutputParams::LATEX;
+			if (conv.latex_flavor() == "xelatex")
 				return OutputParams::XETEX;
-			if (conv.latex_flavor == "lualatex")
+			if (conv.latex_flavor() == "lualatex")
 				return OutputParams::LUATEX;
-			if (conv.latex_flavor == "dvilualatex")
+			if (conv.latex_flavor() == "dvilualatex")
 				return OutputParams::DVILUATEX;
-			if (conv.latex_flavor == "pdflatex")
+			if (conv.latex_flavor() == "pdflatex")
 				return OutputParams::PDFLATEX;
-		if (conv.xml)
+		}
+		if (conv.xml())
 			return OutputParams::XML;
 	}
-	return OutputParams::LATEX;
+	return buffer ? buffer->params().getOutputFlavor()
+		      : OutputParams::LATEX;
+}
+
+
+string Converters::getHyperrefDriver(Graph::EdgePath const & path)
+{
+	for (Graph::EdgePath::const_iterator cit = path.begin();
+	     cit != path.end(); ++cit) {
+		Converter const & conv = converterlist_[*cit];
+		if (!conv.hyperref_driver().empty())
+			return conv.hyperref_driver();
+	}
+	return string();
+}
+
+
+bool Converters::checkAuth(Converter const & conv, string const & doc_fname,
+			   bool use_shell_escape)
+{
+	string conv_command = conv.command();
+	bool const has_shell_escape = contains(conv_command, "-shell-escape")
+				|| contains(conv_command, "-enable-write18");
+	if (conv.latex() && has_shell_escape && !use_shell_escape) {
+		docstring const shellescape_warning =
+		      bformat(_("<p>The following LaTeX backend has been "
+		        "configured to allow execution of external programs "
+		        "for any document:</p>"
+		        "<center><p><tt>%1$s</tt></p></center>"
+		        "<p>This is a dangerous configuration. Please, "
+		        "consider using the support offered by LyX for "
+		        "allowing this privilege only to documents that "
+			"actually need it, instead.</p>"),
+		        from_utf8(conv_command));
+		frontend::Alert::error(_("Security Warning"),
+					shellescape_warning , false);
+	} else if (!conv.latex())
+		use_shell_escape = false;
+	if (!conv.need_auth() && !use_shell_escape)
+		return true;
+	size_t const token_pos = conv_command.find("$$");
+	bool const has_token = token_pos != string::npos;
+	string const command = use_shell_escape && !has_shell_escape
+		? (has_token ? conv_command.insert(token_pos, "-shell-escape ")
+			     : conv_command.append(" -shell-escape"))
+		: conv_command;
+	docstring const security_warning = (use_shell_escape
+	    ? bformat(_("<p>The following LaTeX backend has been requested "
+	        "to allow execution of external programs:</p>"
+	        "<center><p><tt>%1$s</tt></p></center>"
+	        "<p>The external programs can execute arbitrary commands on "
+	        "your system, including dangerous ones, if instructed to do "
+	        "so by a maliciously crafted LyX document.</p>"),
+	      from_utf8(command))
+	    : bformat(_("<p>The requested operation requires the use of a "
+	        "converter from %2$s to %3$s:</p>"
+	        "<blockquote><p><tt>%1$s</tt></p></blockquote>"
+	        "<p>This external program can execute arbitrary commands on "
+	        "your system, including dangerous ones, if instructed to do "
+	        "so by a maliciously crafted LyX document.</p>"),
+	      from_utf8(command), from_utf8(conv.from()),
+	      from_utf8(conv.to())));
+	if (lyxrc.use_converter_needauth_forbidden && !use_shell_escape) {
+		frontend::Alert::error(
+		    _("An external converter is disabled for security reasons"),
+		    security_warning + _(
+		    "<p><b>Your current preference settings forbid its execution.</b></p>"
+		    "<p>(To change this setting, go to <i>Preferences &#x25b9; File "
+		    "Handling &#x25b9; Converters</i> and uncheck <i>Security &#x25b9; "
+		    "Forbid needauth converters</i>.)"), false);
+		return false;
+	}
+	if (!lyxrc.use_converter_needauth && !use_shell_escape)
+		return true;
+	docstring const security_title = use_shell_escape
+		? _("A LaTeX backend requires your authorization")
+		: _("An external converter requires your authorization");
+	int choice;
+	docstring const security_warning2 = security_warning + (use_shell_escape
+		? _("<p>Should LaTeX backends be allowed to run external "
+		    "programs?</p><p><b>Allow them only if you trust the "
+		    "origin/sender of the LyX document!</b></p>")
+		: _("<p>Would you like to run this converter?</p>"
+		    "<p><b>Only run if you trust the origin/sender of the LyX "
+		    "document!</b></p>"));
+	docstring const no = use_shell_escape
+				? _("Do &not allow") : _("Do &not run");
+	docstring const yes = use_shell_escape ? _("A&llow") : _("&Run");
+	docstring const always = use_shell_escape
+					? _("&Always allow for this document")
+					: _("&Always run for this document");
+	if (!doc_fname.empty()) {
+		LYXERR(Debug::FILES, "looking up: " << doc_fname);
+		bool authorized = use_shell_escape
+			? theSession().shellescapeFiles().findAuth(doc_fname)
+			: theSession().authFiles().find(doc_fname);
+		if (!authorized) {
+			choice = frontend::Alert::prompt(security_title,
+							 security_warning2,
+							 0, 0, no, yes, always);
+			if (choice == 2) {
+				if (use_shell_escape)
+					theSession().shellescapeFiles().insert(doc_fname, true);
+				else
+					theSession().authFiles().insert(doc_fname);
+			}
+		} else {
+			choice = 1;
+		}
+	} else {
+		choice = frontend::Alert::prompt(security_title,
+						 security_warning2,
+						 0, 0, no, yes);
+	}
+	return choice != 0;
 }
 
 
@@ -297,20 +418,21 @@ bool Converters::convert(Buffer const * buffer,
 			// default one from ImageMagic.
 			string const from_ext = from_format.empty() ?
 				getExtension(from_file.absFileName()) :
-				formats.extension(from_format);
-			string const to_ext = formats.extension(to_format);
+				theFormats().extension(from_format);
+			string const to_ext = theFormats().extension(to_format);
 			string const command =
 				os::python() + ' ' +
 				quoteName(libFileSearch("scripts", "convertDefault.py").toFilesystemEncoding()) +
-				' ' +
-				quoteName(from_ext + ':' + from_file.toFilesystemEncoding()) +
-				' ' +
-				quoteName(to_ext + ':' + to_file.toFilesystemEncoding());
+				' ' + from_ext + ' ' +
+				quoteName(from_file.toFilesystemEncoding()) +
+				' ' + to_ext + ' ' +
+				quoteName(to_file.toFilesystemEncoding());
 			LYXERR(Debug::FILES, "No converter defined! "
 				   "I use convertDefault.py:\n\t" << command);
 			Systemcall one;
-			one.startscript(Systemcall::Wait, command, buffer ?
-					buffer->filePath() : string());
+			one.startscript(Systemcall::Wait, command,
+			                buffer ? buffer->filePath() : string(),
+			                buffer ? buffer->layoutPos() : string());
 			if (to_file.isReadableFile()) {
 				if (conversionflags & try_cache)
 					ConverterCache::get().add(orig_from,
@@ -337,15 +459,21 @@ bool Converters::convert(Buffer const * buffer,
 	// buffer is only invalid for importing, and then runparams is not
 	// used anyway.
 	OutputParams runparams(buffer ? &buffer->params().encoding() : 0);
-	runparams.flavor = getFlavor(edgepath);
+	runparams.flavor = getFlavor(edgepath, buffer);
 
 	if (buffer) {
-		runparams.use_japanese = buffer->params().bufferFormat() == "platex";
+		runparams.use_japanese =
+			(buffer->params().bufferFormat() == "latex"
+			 || suffixIs(buffer->params().bufferFormat(), "-ja"))
+			&& buffer->params().encoding().package() == Encoding::japanese;
 		runparams.use_indices = buffer->params().use_indices;
-		runparams.bibtex_command = (buffer->params().bibtex_command == "default") ?
-			string() : buffer->params().bibtex_command;
+		runparams.bibtex_command = buffer->params().bibtexCommand();
 		runparams.index_command = (buffer->params().index_command == "default") ?
 			string() : buffer->params().index_command;
+		runparams.document_language = buffer->params().language->babel();
+		runparams.only_childbibs = !buffer->params().useBiblatex()
+				&& !buffer->params().useBibtopic()
+				&& buffer->params().multibib == "child";
 	}
 
 	// Some converters (e.g. lilypond) can only output files to the
@@ -353,6 +481,8 @@ bool Converters::convert(Buffer const * buffer,
 	// This has the added benefit that all other files that may be
 	// generated by the converter are deleted when LyX closes and do not
 	// clutter the real working directory.
+	// FIXME: This does not work if path is an UNC path on windows
+	//        (bug 6127).
 	string const path(onlyPath(from_file.absFileName()));
 	// Prevent the compiler from optimizing away p
 	FileName pp(path);
@@ -369,22 +499,25 @@ bool Converters::convert(Buffer const * buffer,
 	for (Graph::EdgePath::const_iterator cit = edgepath.begin();
 	     cit != edgepath.end(); ++cit) {
 		Converter const & conv = converterlist_[*cit];
-		bool dummy = conv.To->dummy() && conv.to != "program";
+		bool dummy = conv.To()->dummy() && conv.to() != "program";
 		if (!dummy) {
 			LYXERR(Debug::FILES, "Converting from  "
-			       << conv.from << " to " << conv.to);
+			       << conv.from() << " to " << conv.to());
 		}
 		infile = outfile;
-		outfile = FileName(conv.result_dir.empty()
-			? changeExtension(from_file.absFileName(), conv.To->extension())
-			: addName(subst(conv.result_dir,
+		outfile = FileName(conv.result_file().empty()
+			? changeExtension(from_file.absFileName(), conv.To()->extension())
+			: addName(subst(conv.result_dir(),
 					token_base, from_base),
-				  subst(conv.result_file,
+				  subst(conv.result_file(),
 					token_base, onlyFileName(from_base))));
 
 		// if input and output files are equal, we use a
 		// temporary file as intermediary (JMarc)
 		FileName real_outfile;
+		if (!conv.result_file().empty())
+			real_outfile = FileName(changeExtension(from_file.absFileName(),
+				conv.To()->extension()));
 		if (outfile == infile) {
 			real_outfile = infile;
 			// when importing, a buffer does not necessarily exist
@@ -395,21 +528,88 @@ bool Converters::convert(Buffer const * buffer,
 						   "tmpfile.out"));
 		}
 
-		if (conv.latex) {
+		if (buffer && buffer->params().use_minted
+		    && lyxrc.pygmentize_command.empty() && conv.latex()) {
+			bool dowarn = false;
+			// Warn only if listings insets are actually used
+			for (Paragraph const & par : buffer->paragraphs()) {
+				InsetList const & insets = par.insetList();
+				pos_type lstpos = insets.find(LISTINGS_CODE, 0);
+				pos_type incpos = insets.find(INCLUDE_CODE, 0);
+				if (incpos >= 0) {
+					InsetInclude const * include =
+						static_cast<InsetInclude *>
+						        (insets.get(incpos));
+					if (include->params().getCmdName() !=
+					                        "inputminted") {
+						incpos = -1;
+					}
+				}
+				if (lstpos >= 0 || incpos >= 0) {
+					dowarn = true;
+					break;
+				}
+			}
+			if (dowarn) {
+				Alert::warning(_("Pygments driver command not found!"),
+				    _("The driver command necessary to use the minted package\n"
+				      "(pygmentize) has not been found. Make sure you have\n"
+				      "the python-pygments module installed or, if the driver\n"
+				      "is named differently, to add the following line to the\n"
+				      "document preamble:\n\n"
+				      "\\AtBeginDocument{\\renewcommand{\\MintedPygmentize}{driver}}\n\n"
+				      "where 'driver' is name of the driver command."));
+			}
+		}
+
+		if (!checkAuth(conv, buffer ? buffer->absFileName() : string(),
+			       buffer && buffer->params().shell_escape))
+			return false;
+
+		if (conv.latex()) {
+			// We are not importing, we have a buffer
+			LATTEST(buffer);
 			run_latex = true;
-			string const command = subst(conv.command, token_from, "");
+			string command = conv.command();
+			command = subst(command, token_from, "");
+			command = subst(command, token_latex_encoding,
+			                buffer->params().encoding().latexName());
+			if (buffer->params().shell_escape
+			    && !contains(command, "-shell-escape"))
+				command += " -shell-escape ";
 			LYXERR(Debug::FILES, "Running " << command);
 			if (!runLaTeX(*buffer, command, runparams, errorList))
 				return false;
 		} else {
-			if (conv.need_aux && !run_latex
-			    && !latex_command_.empty()) {
-				string const command = (buffer && buffer->params().useNonTeXFonts) ?
-					xelatex_command_ : latex_command_;
-				LYXERR(Debug::FILES, "Running " << command
-					<< " to update aux file");
-				if (!runLaTeX(*buffer, command, runparams, errorList))
-					return false;
+			if (conv.need_aux() && !run_latex) {
+				// We are not importing, we have a buffer
+				LATTEST(buffer);
+				string command;
+				switch (runparams.flavor) {
+				case OutputParams::DVILUATEX:
+					command = dvilualatex_command_;
+					break;
+				case OutputParams::LUATEX:
+					command = lualatex_command_;
+					break;
+				case OutputParams::PDFLATEX:
+					command = pdflatex_command_;
+					break;
+				case OutputParams::XETEX:
+					command = xelatex_command_;
+					break;
+				default:
+					command = latex_command_;
+					break;
+				}
+				if (!command.empty()) {
+					LYXERR(Debug::FILES, "Running "
+						<< command
+						<< " to update aux file");
+					if (!runLaTeX(*buffer, command,
+						      runparams, errorList))
+						return false;
+				}
 			}
 
 			// FIXME UNICODE
@@ -418,22 +618,24 @@ bool Converters::convert(Buffer const * buffer,
 			string const outfile2 =
 				to_utf8(makeRelPath(from_utf8(outfile.absFileName()), from_utf8(path)));
 
-			string command = conv.command;
+			string command = conv.command();
 			command = subst(command, token_from, quoteName(infile2));
 			command = subst(command, token_base, quoteName(from_base));
 			command = subst(command, token_to, quoteName(outfile2));
 			command = subst(command, token_path, quoteName(onlyPath(infile.absFileName())));
 			command = subst(command, token_orig_path, quoteName(onlyPath(orig_from.absFileName())));
+			command = subst(command, token_orig_from, quoteName(onlyFileName(orig_from.absFileName())));
 			command = subst(command, token_encoding, buffer ? buffer->params().encoding().iconvName() : string());
-			command = libScriptSearch(command);
 
-			if (!conv.parselog.empty())
+			if (!conv.parselog().empty())
 				command += " 2> " + quoteName(infile2 + ".out");
 
-			if (conv.from == "dvi" && conv.to == "ps")
+			// it is not actually not necessary to test for buffer here,
+			// but it pleases coverity.
+			if (buffer && conv.from() == "dvi" && conv.to() == "ps")
 				command = add_options(command,
 						      buffer->params().dvips_options());
-			else if (conv.from == "dvi" && prefixIs(conv.to, "pdf"))
+			else if (buffer && conv.from() == "dvi" && prefixIs(conv.to(), "pdf"))
 				command = add_options(command,
 						      dvipdfm_options(buffer->params()));
 
@@ -447,16 +649,19 @@ bool Converters::convert(Buffer const * buffer,
 			if (dummy) {
 				res = one.startscript(Systemcall::DontWait,
 					to_filesystem8bit(from_utf8(command)),
-					buffer ? buffer->filePath() : string());
+					buffer ? buffer->filePath() : string(),
+					buffer ? buffer->layoutPos() : string());
 				// We're not waiting for the result, so we can't do anything
 				// else here.
 			} else {
 				res = one.startscript(Systemcall::Wait,
 						to_filesystem8bit(from_utf8(command)),
 						buffer ? buffer->filePath()
+						       : string(),
+						buffer ? buffer->layoutPos()
 						       : string());
 				if (!real_outfile.empty()) {
-					Mover const & mover = getMover(conv.to);
+					Mover const & mover = getMover(conv.to());
 					if (!mover.rename(outfile, real_outfile))
 						res = -1;
 					else
@@ -466,23 +671,23 @@ bool Converters::convert(Buffer const * buffer,
 					// converters to use the renamed file...
 					outfile = real_outfile;
 				}
-  
-				if (!conv.parselog.empty()) {
+
+				if (!conv.parselog().empty()) {
 					string const logfile =  infile2 + ".log";
-					string const script = libScriptSearch(conv.parselog);
-					string const command2 = script +
+					string const command2 = conv.parselog() +
 						" < " + quoteName(infile2 + ".out") +
 						" > " + quoteName(logfile);
 					one.startscript(Systemcall::Wait,
 						to_filesystem8bit(from_utf8(command2)),
-						buffer->filePath());
+						buffer->filePath(),
+						buffer->layoutPos());
 					if (!scanLog(*buffer, command, makeAbsPath(logfile, path), errorList))
 						return false;
 				}
 			}
 
 			if (res) {
-				if (conv.to == "program") {
+				if (conv.to() == "program") {
 					Alert::error(_("Build errors"),
 						_("There were errors during the build process."));
 				} else {
@@ -498,18 +703,18 @@ bool Converters::convert(Buffer const * buffer,
 	}
 
 	Converter const & conv = converterlist_[edgepath.back()];
-	if (conv.To->dummy())
+	if (conv.To()->dummy())
 		return true;
 
-	if (!conv.result_dir.empty()) {
+	if (!conv.result_dir().empty()) {
 		// The converter has put the file(s) in a directory.
 		// In this case we ignore the given to_file.
 		if (from_base != to_base) {
-			string const from = subst(conv.result_dir,
+			string const from = subst(conv.result_dir(),
 					    token_base, from_base);
-			string const to = subst(conv.result_dir,
+			string const to = subst(conv.result_dir(),
 					  token_base, to_base);
-			Mover const & mover = getMover(conv.from);
+			Mover const & mover = getMover(conv.from());
 			if (!mover.rename(FileName(from), FileName(to))) {
 				Alert::error(_("Cannot convert file"),
 					bformat(_("Could not move a temporary directory from %1$s to %2$s."),
@@ -521,7 +726,7 @@ bool Converters::convert(Buffer const * buffer,
 	} else {
 		if (conversionflags & try_cache)
 			ConverterCache::get().add(orig_from, to_format, outfile);
-		return move(conv.to, outfile, to_file, conv.latex);
+		return move(conv.to(), outfile, to_file, conv.latex());
 	}
 }
 
@@ -572,7 +777,7 @@ bool Converters::formatIsUsed(string const & format)
 	ConverterList::const_iterator cit = converterlist_.begin();
 	ConverterList::const_iterator end = converterlist_.end();
 	for (; cit != end; ++cit) {
-		if (cit->from == format || cit->to == format)
+		if (cit->from() == format || cit->to() == format)
 			return true;
 	}
 	return false;
@@ -595,50 +800,50 @@ bool Converters::scanLog(Buffer const & buffer, string const & /*command*/,
 }
 
 
-namespace {
-
-class ShowMessage
-	: public boost::signals::trackable {
-public:
-	ShowMessage(Buffer const & b) : buffer_(b) {}
-	void operator()(docstring const & msg) const { buffer_.message(msg); }
-private:
-	Buffer const & buffer_;
-};
-
-}
-
-
 bool Converters::runLaTeX(Buffer const & buffer, string const & command,
 			  OutputParams const & runparams, ErrorList & errorList)
 {
 	buffer.setBusy(true);
 	buffer.message(_("Running LaTeX..."));
 
-	runparams.document_language = buffer.params().language->babel();
-
 	// do the LaTeX run(s)
 	string const name = buffer.latexName();
 	LaTeX latex(command, runparams, FileName(makeAbsPath(name)),
-		    buffer.filePath());
+	            buffer.filePath(), buffer.layoutPos(),
+	            buffer.lastPreviewError());
 	TeXErrors terr;
-	ShowMessage show(buffer);
-	latex.message.connect(show);
+	// The connection closes itself at the end of the scope when latex is
+	// destroyed. One cannot close (and destroy) buffer while the converter is
+	// running.
+	latex.message.connect([&buffer](docstring const & msg){
+			buffer.message(msg);
+		});
 	int const result = latex.run(terr);
 
 	if (result & LaTeX::ERRORS)
 		buffer.bufferErrors(terr, errorList);
 
-	// check return value from latex.run().
-	if ((result & LaTeX::NO_LOGFILE) && !buffer.isClone()) {
+	if (!errorList.empty()) {
+	  // We will show the LaTeX Errors GUI later which contains
+	  // specific error messages so it would be repetitive to give
+	  // e.g. the "finished with an error" dialog in addition.
+	}
+	else if (result & LaTeX::NO_LOGFILE) {
 		docstring const str =
 			bformat(_("LaTeX did not run successfully. "
 					       "Additionally, LyX could not locate "
 					       "the LaTeX log %1$s."), from_utf8(name));
 		Alert::error(_("LaTeX failed"), str);
-	} else if ((result & LaTeX::NO_OUTPUT) && !buffer.isClone()) {
+	} else if (result & LaTeX::NONZERO_ERROR) {
+		docstring const str =
+			bformat(_( "The external program\n%1$s\n"
+			      "finished with an error. "
+			      "It is recommended you fix the cause of the external "
+			      "program's error (check the logs). "), from_utf8(command));
+		Alert::error(_("LaTeX failed"), str);
+	} else if (result & LaTeX::NO_OUTPUT) {
 		Alert::warning(_("Output is empty"),
-			       _("An empty output file was generated."));
+			       _("No output file was generated."));
 	}
 
 
@@ -650,7 +855,6 @@ bool Converters::runLaTeX(Buffer const & buffer, string const & command,
 			LaTeX::NO_OUTPUT;
 
 	return (result & ERROR_MASK) == 0;
-
 }
 
 
@@ -658,58 +862,59 @@ bool Converters::runLaTeX(Buffer const & buffer, string const & command,
 void Converters::buildGraph()
 {
 	// clear graph's data structures
-	G_.init(formats.size());
+	G_.init(theFormats().size());
 	// each of the converters knows how to convert one format to another
-	// so, for each of them, we create an arrow on the graph, going from 
+	// so, for each of them, we create an arrow on the graph, going from
 	// the one to the other
 	ConverterList::iterator it = converterlist_.begin();
 	ConverterList::iterator const end = converterlist_.end();
 	for (; it != end ; ++it) {
-		int const from = formats.getNumber(it->from);
-		int const to   = formats.getNumber(it->to);
+		int const from = theFormats().getNumber(it->from());
+		int const to   = theFormats().getNumber(it->to());
+		LASSERT(from >= 0, continue);
+		LASSERT(to >= 0, continue);
 		G_.addEdge(from, to);
 	}
 }
 
 
-vector<Format const *> const
-Converters::intToFormat(vector<int> const & input)
+FormatList const Converters::intToFormat(vector<int> const & input)
 {
-	vector<Format const *> result(input.size());
+	FormatList result(input.size());
 
 	vector<int>::const_iterator it = input.begin();
 	vector<int>::const_iterator const end = input.end();
-	vector<Format const *>::iterator rit = result.begin();
+	FormatList::iterator rit = result.begin();
 	for ( ; it != end; ++it, ++rit) {
-		*rit = &formats.get(*it);
+		*rit = &theFormats().get(*it);
 	}
 	return result;
 }
 
 
-vector<Format const *> const
-Converters::getReachableTo(string const & target, bool const clear_visited)
+FormatList const Converters::getReachableTo(string const & target,
+		bool const clear_visited)
 {
 	vector<int> const & reachablesto =
-		G_.getReachableTo(formats.getNumber(target), clear_visited);
+		G_.getReachableTo(theFormats().getNumber(target), clear_visited);
 
 	return intToFormat(reachablesto);
 }
 
 
-vector<Format const *> const
-Converters::getReachable(string const & from, bool const only_viewable,
-			 bool const clear_visited, set<string> const & excludes)
+FormatList const Converters::getReachable(string const & from,
+		bool const only_viewable, bool const clear_visited,
+		set<string> const & excludes)
 {
-	set<int> excluded_numbers;;
+	set<int> excluded_numbers;
 
 	set<string>::const_iterator sit = excludes.begin();
 	set<string>::const_iterator const end = excludes.end();
 	for (; sit != end; ++sit)
-		excluded_numbers.insert(formats.getNumber(*sit));
+		excluded_numbers.insert(theFormats().getNumber(*sit));
 
 	vector<int> const & reachables =
-		G_.getReachable(formats.getNumber(from),
+		G_.getReachable(theFormats().getNumber(from),
 				only_viewable,
 				clear_visited,
 				excluded_numbers);
@@ -720,41 +925,40 @@ Converters::getReachable(string const & from, bool const only_viewable,
 
 bool Converters::isReachable(string const & from, string const & to)
 {
-	return G_.isReachable(formats.getNumber(from),
-			      formats.getNumber(to));
+	return G_.isReachable(theFormats().getNumber(from),
+			      theFormats().getNumber(to));
 }
 
 
 Graph::EdgePath Converters::getPath(string const & from, string const & to)
 {
-	return G_.getPath(formats.getNumber(from),
-			  formats.getNumber(to));
+	return G_.getPath(theFormats().getNumber(from),
+			  theFormats().getNumber(to));
 }
 
 
-vector<Format const *> Converters::importableFormats()
+FormatList Converters::importableFormats()
 {
 	vector<string> l = loaders();
-	vector<Format const *> result = getReachableTo(l[0], true);
+	FormatList result = getReachableTo(l[0], true);
 	vector<string>::const_iterator it = l.begin() + 1;
 	vector<string>::const_iterator en = l.end();
 	for (; it != en; ++it) {
-		vector<Format const *> r = getReachableTo(*it, false);
+		FormatList r = getReachableTo(*it, false);
 		result.insert(result.end(), r.begin(), r.end());
 	}
 	return result;
 }
 
 
-vector<Format const *> Converters::exportableFormats(bool only_viewable)
+FormatList Converters::exportableFormats(bool only_viewable)
 {
 	vector<string> s = savers();
-	vector<Format const *> result = getReachable(s[0], only_viewable, true);
+	FormatList result = getReachable(s[0], only_viewable, true);
 	vector<string>::const_iterator it = s.begin() + 1;
 	vector<string>::const_iterator en = s.end();
 	for (; it != en; ++it) {
-		vector<Format const *> r =
-			getReachable(*it, only_viewable, false);
+		 FormatList r = getReachable(*it, only_viewable, false);
 		result.insert(result.end(), r.begin(), r.end());
 	}
 	return result;
