@@ -13,12 +13,12 @@
 #ifndef PAINTER_H
 #define PAINTER_H
 
-#include "Color.h"
-
 #include "support/strfwd.h"
+#include "support/types.h"
 
 namespace lyx {
 
+class Font;
 class FontInfo;
 
 namespace graphics { class Image; }
@@ -38,12 +38,7 @@ namespace frontend {
  *
  * The intention for a toolkit is that it uses these methods to paint
  * onto a backing pixmap. Only when expose events arrive via the event
- * queue (perhaps generated via Screen::expose), does the copy onto
- * the actual WorkArea widget take place. Paints are wrapped in (possibly
- * recursive) calls to start() and end() to facilitate the backing pixmap
- * management.
- *
- * Note that the methods return *this for convenience.
+ * queue, does the copy onto the actual WorkArea widget take place.
  *
  * Caution: All char_type and docstring arguments of the text drawing
  * methods of this class are no UCS4 chars or strings if the font is a
@@ -54,17 +49,25 @@ namespace frontend {
  */
 class Painter {
 public:
-	Painter() : drawing_enabled_(true) {}
+	Painter(double pixel_ratio) : pixel_ratio_(pixel_ratio) {}
 
-	float line_width;
-	static const float thin_line;
+	static const int thin_line;
 
 	/// possible line styles
 	enum line_style {
 		line_solid, //< solid line
+		line_solid_aliased, //< solid line, no anti-aliasing (used as a
+		                    // workaround to painting issues)
 		line_onoffdash //< dashes with spaces
 	};
 
+	/// possible fill styles
+	enum fill_style {
+		fill_none,
+		fill_oddeven,
+		fill_winding
+	};
+
 	/// possible character styles of preedit string.
 	/// This is used for CJK input method support.
 	enum preedit_style {
@@ -77,7 +80,7 @@ public:
 
 	/// draw a line from point to point
 	virtual void line(int x1, int y1, int x2, int y2, Color,
-		line_style = line_solid, float line_width = thin_line) = 0;
+		line_style = line_solid, int line_width = thin_line) = 0;
 
 	/**
 	 * lines -  draw a set of lines
@@ -86,11 +89,29 @@ public:
 	 * @param np size of the points array
 	 */
 	virtual void lines(int const * xp, int const * yp, int np, Color,
-		line_style = line_solid, float line_width = thin_line) = 0;
+		fill_style = fill_none, line_style = line_solid,
+		int line_width = thin_line) = 0;
+
+	/**
+	 * path -  draw a path with bezier curves
+	 * @param xp array of points' x co-ords
+	 * @param yp array of points' y co-ords
+	 * @param c1x array of first control points' x co-ords
+	 * @param c1y array of first control points' y co-ords
+	 * @param c2x array of second control points' x co-ords
+	 * @param c2y array of second control points' y co-ords
+	 * @param np size of the points array
+	 */
+	virtual void path(int const * xp, int const * yp,
+		int const * c1x, int const * c1y,
+		int const * c2x, int const * c2y,
+		int np, Color,
+		fill_style = fill_none, line_style = line_solid,
+		int line_width = thin_line) = 0;
 
 	/// draw a rectangle
 	virtual void rectangle(int x, int y, int w, int h, Color,
-		line_style = line_solid, float line_width = thin_line) = 0;
+		line_style = line_solid, int line_width = thin_line) = 0;
 
 	/// draw a filled rectangle
 	virtual void fillRectangle(int x, int y, int w, int h, Color) = 0;
@@ -102,30 +123,37 @@ public:
 	/// draw a pixel
 	virtual void point(int x, int y, Color) = 0;
 
-	/// draw a filled rectangle with the shape of a 3D button
-	virtual void button(int x, int y, int w, int h, bool mouseHover) = 0;
-
 	/// draw an image from the image cache
 	virtual void image(int x, int y, int w, int h,
 		graphics::Image const & image) = 0;
 
-	/// draw a string at position x, y (y is the baseline)
-	/**
-	* \return the width of the drawn text.
-	*/
-	virtual int text(int x, int y, docstring const & str, FontInfo const & f) = 0;
-
-	void setDrawingEnabled(bool drawing_enabled)
-	{ drawing_enabled_ = drawing_enabled; }
-
-	/// Indicate wether real screen drawing shall be done or not.
-	bool isDrawingEnabled() const { return drawing_enabled_; }
+	/// draw a string at position x, y (y is the baseline).
+	virtual void text(int x, int y, docstring const & str, FontInfo const & f) = 0;
 
 	/// draw a char at position x, y (y is the baseline)
-	/**
-	* \return the width of the drawn text.
-	*/
-	virtual int text(int x, int y, char_type c, FontInfo const & f) = 0;
+	virtual void text(int x, int y, char_type c, FontInfo const & f) = 0;
+
+	/** draw a string at position x, y (y is the baseline). The
+	 * text direction is enforced by the \c Font.
+	 */
+	virtual void text(int x, int y, docstring const & str, Font const & f,
+                      double wordspacing, double textwidth) = 0;
+
+	/** draw a string at position x, y (y is the baseline), but
+	 * make sure that the part between \c from and \c to is in
+	 * \c other color. The text direction is enforced by the \c Font.
+	 */
+	virtual void text(int x, int y, docstring const & str, Font const & f,
+	                  Color other, size_type from, size_type to,
+                      double wordspacing, double textwidth) = 0;
+
+	// Returns true if the painter does not actually paint.
+	virtual bool isNull() const = 0;
+
+	double pixelRatio() const { return pixel_ratio_; }
+
+	/// draw the underbar, strikeout, xout, uuline and uwave font attributes
+	virtual void textDecoration(FontInfo const & f, int x, int y, int width) = 0;
 
 	/**
 	 * Draw a string and enclose it inside a rectangle. If
@@ -138,22 +166,22 @@ public:
 
 	/// draw a string and enclose it inside a button frame
 	virtual void buttonText(int x, int baseline, docstring const & s,
-		FontInfo const & font, bool mouseHover) = 0;
+		FontInfo const & font, Color back, Color frame, int offset) = 0;
 
 	/// draw a character of a preedit string for cjk support.
 	virtual int preeditText(int x, int y,
 		char_type c, FontInfo const & f, preedit_style style) = 0;
 
 	/// start monochrome painting mode, i.e. map every color into [min,max]
-	virtual void enterMonochromeMode(Color const & min, 
+	virtual void enterMonochromeMode(Color const & min,
 		Color const & max) = 0;
 	/// leave monochrome painting mode
 	virtual void leaveMonochromeMode() = 0;
 	/// draws a wavy line that can be used for underlining.
 	virtual void wavyHorizontalLine(int x, int y, int width, ColorCode col) = 0;
 private:
-	///
-	bool drawing_enabled_;
+	/// Ratio between physical pixels and device-independent pixels
+	double pixel_ratio_;
 };
 
 } // namespace frontend
