@@ -6,7 +6,7 @@
  * \author Richard Heck
  * \author Et Alia
  *
- * Some of the material in this file previously appeared in 
+ * Some of the material in this file previously appeared in
  * GuiCitationDialog.cpp.
  *
  * Full author contact details are available in file CREDITS.
@@ -15,9 +15,11 @@
 #include <config.h>
 
 #include "GuiSelectionManager.h"
+#include "qt_helpers.h"
 
 #include "support/debug.h"
 
+#include <QAbstractItemModel>
 #include <QAbstractListModel>
 #include <QItemSelection>
 #include <QListView>
@@ -40,23 +42,26 @@
 namespace lyx {
 namespace frontend {
 
-GuiSelectionManager::GuiSelectionManager(
-	QAbstractItemView * avail,
-	QListView * sel,
-	QPushButton * add, 
-	QPushButton * del, 
-	QPushButton * up, 
-	QPushButton * down,
-	QAbstractListModel * amod,
-	QAbstractListModel * smod)
-  : availableLV(avail), selectedLV(sel), addPB(add), deletePB(del),
-		upPB(up), downPB(down), availableModel(amod), selectedModel(smod),
-    selectedHasFocus_(false)
+GuiSelectionManager::GuiSelectionManager(QObject * parent,
+                                         QAbstractItemView * avail,
+                                         QAbstractItemView * sel,
+                                         QPushButton * add,
+                                         QPushButton * del,
+                                         QPushButton * up,
+                                         QPushButton * down,
+                                         QAbstractListModel * amod,
+                                         QAbstractItemModel * smod,
+                                         int const main_sel_col)
+: QObject(parent), availableLV(avail), selectedLV(sel),
+  addPB(add), deletePB(del), upPB(up), downPB(down),
+  availableModel(amod), selectedModel(smod),
+  selectedHasFocus_(false), main_sel_col_(main_sel_col)
 {
-	
 	selectedLV->setModel(smod);
 	availableLV->setModel(amod);
-	
+	selectedLV->setSelectionBehavior(QAbstractItemView::SelectRows);
+	selectedLV->setSelectionMode(QAbstractItemView::SingleSelection);
+
 	connect(availableLV->selectionModel(),
 	        SIGNAL(currentChanged(QModelIndex, QModelIndex)),
 	        this, SLOT(availableChanged(QModelIndex, QModelIndex)));
@@ -69,17 +74,19 @@ GuiSelectionManager::GuiSelectionManager(
 	connect(selectedLV->selectionModel(),
 	        SIGNAL(selectionChanged(QItemSelection, QItemSelection)),
 	        this, SLOT(selectedChanged(QItemSelection, QItemSelection)));
-	connect(addPB, SIGNAL(clicked()), 
+	connect(selectedLV->itemDelegate(), SIGNAL(commitData(QWidget*)),
+		this, SLOT(selectedEdited()));
+	connect(addPB, SIGNAL(clicked()),
 	        this, SLOT(addPB_clicked()));
-	connect(deletePB, SIGNAL(clicked()), 
+	connect(deletePB, SIGNAL(clicked()),
 	        this, SLOT(deletePB_clicked()));
-	connect(upPB, SIGNAL(clicked()), 
+	connect(upPB, SIGNAL(clicked()),
 	        this, SLOT(upPB_clicked()));
-	connect(downPB, SIGNAL(clicked()), 
+	connect(downPB, SIGNAL(clicked()),
 	        this, SLOT(downPB_clicked()));
-	connect(availableLV, SIGNAL(doubleClicked(QModelIndex)), 
+	connect(availableLV, SIGNAL(doubleClicked(QModelIndex)),
 	        this, SLOT(availableLV_doubleClicked(QModelIndex)));
-	
+
 	availableLV->installEventFilter(this);
 	selectedLV->installEventFilter(this);
 }
@@ -94,19 +101,19 @@ void GuiSelectionManager::update()
 }
 
 
-QModelIndex GuiSelectionManager::getSelectedIndex() const
+QModelIndex GuiSelectionManager::getSelectedIndex(int const c) const
 {
 	QModelIndexList avail = availableLV->selectionModel()->selectedIndexes();
-	QModelIndexList sel   = selectedLV->selectionModel()->selectedIndexes();
+	QModelIndexList sel   = selectedLV->selectionModel()->selectedRows(c);
 	bool const have_avl = !avail.isEmpty();
 	bool const have_sel = !sel.isEmpty();
 
-	if (selectedFocused()) { 
+	if (selectedFocused()) {
 		if (have_sel)
 			return sel.front();
 		if (have_avl)
 			return avail.front();
-	} 
+	}
 	else { // available has focus
 		if (have_avl)
 			return avail.front();
@@ -120,7 +127,7 @@ QModelIndex GuiSelectionManager::getSelectedIndex() const
 void GuiSelectionManager::updateAddPB()
 {
 	int const arows = availableModel->rowCount();
-	QModelIndexList const availSels = 
+	QModelIndexList const availSels =
 		availableLV->selectionModel()->selectedIndexes();
 	addPB->setEnabled(arows > 0 &&
 		!availSels.isEmpty() &&
@@ -135,9 +142,9 @@ void GuiSelectionManager::updateDelPB()
 		deletePB->setEnabled(false);
 		return;
 	}
-	QModelIndexList const selSels = 
+	QModelIndexList const selSels =
 		selectedLV->selectionModel()->selectedIndexes();
-	int const sel_nr = 	selSels.empty() ? -1 : selSels.first().row();
+	int const sel_nr = selSels.empty() ? -1 : selSels.first().row();
 	deletePB->setEnabled(sel_nr >= 0);
 }
 
@@ -149,9 +156,9 @@ void GuiSelectionManager::updateUpPB()
 		upPB->setEnabled(false);
 		return;
 	}
-	QModelIndexList const selSels = 
+	QModelIndexList const selSels =
 			selectedLV->selectionModel()->selectedIndexes();
-	int const sel_nr = 	selSels.empty() ? -1 : selSels.first().row();
+	int const sel_nr = selSels.empty() ? -1 : selSels.first().row();
 	upPB->setEnabled(sel_nr > 0);
 }
 
@@ -163,9 +170,9 @@ void GuiSelectionManager::updateDownPB()
 		downPB->setEnabled(false);
 		return;
 	}
-	QModelIndexList const selSels = 
+	QModelIndexList const selSels =
 			selectedLV->selectionModel()->selectedIndexes();
-	int const sel_nr = 	selSels.empty() ? -1 : selSels.first().row();
+	int const sel_nr = selSels.empty() ? -1 : selSels.first().row();
 	downPB->setEnabled(sel_nr >= 0 && sel_nr < srows - 1);
 }
 
@@ -175,8 +182,8 @@ bool GuiSelectionManager::isSelected(const QModelIndex & idx)
 	if (selectedModel->rowCount() == 0)
 		return false;
 	QVariant const & str = availableModel->data(idx, Qt::DisplayRole);
-	QModelIndexList qmil = 
-			selectedModel->match(selectedModel->index(0), 
+	QModelIndexList qmil =
+			selectedModel->match(selectedModel->index(0, main_sel_col_),
 			                     Qt::DisplayRole, str, 1,
 			                     Qt::MatchFlags(Qt::MatchExactly | Qt::MatchWrap));
 	return !qmil.empty();
@@ -186,7 +193,7 @@ bool GuiSelectionManager::isSelected(const QModelIndex & idx)
 void GuiSelectionManager::availableChanged(QItemSelection const & qis, QItemSelection const &)
 {
 	QModelIndexList il = qis.indexes();
-	if (il.empty())	
+	if (il.empty())
 		return;
 	availableChanged(il.front(), QModelIndex());
 }
@@ -196,7 +203,7 @@ void GuiSelectionManager::availableChanged(const QModelIndex & idx, const QModel
 {
 	if (!idx.isValid())
 		return;
-	
+
 	selectedHasFocus_ = false;
 	updateHook();
 }
@@ -205,7 +212,7 @@ void GuiSelectionManager::availableChanged(const QModelIndex & idx, const QModel
 void GuiSelectionManager::selectedChanged(QItemSelection const & qis, QItemSelection const &)
 {
 	QModelIndexList il = qis.indexes();
-	if (il.empty())	
+	if (il.empty())
 		return;
 	selectedChanged(il.front(), QModelIndex());
 }
@@ -215,13 +222,19 @@ void GuiSelectionManager::selectedChanged(const QModelIndex & idx, const QModelI
 {
 	if (!idx.isValid())
 		return;
-	
+
 	selectedHasFocus_ = true;
 	updateHook();
 }
 
 
-bool GuiSelectionManager::insertRowToSelected(int i, 
+void GuiSelectionManager::selectedEdited()
+{
+	selectionChanged();
+}
+
+
+bool GuiSelectionManager::insertRowToSelected(int i,
 		QMap<int, QVariant> const & itemData)
 {
 	if (i <= -1)
@@ -230,7 +243,23 @@ bool GuiSelectionManager::insertRowToSelected(int i,
 		i = selectedModel->rowCount();
 	if (!selectedModel->insertRow(i))
 		return false;
-	return selectedModel->setItemData(selectedModel->index(i), itemData);
+	return selectedModel->setItemData(selectedModel->index(i, main_sel_col_), itemData);
+}
+
+
+bool GuiSelectionManager::insertRowToSelected(int i, QMap<int, QMap<int, QVariant>> & qms)
+{
+	if (i <= -1)
+		i = 0;
+	if (i > selectedModel->rowCount())
+		i = selectedModel->rowCount();
+	if (!selectedModel->insertRow(i))
+		return false;
+	bool res = true;
+	QMap<int, QMap<int, QVariant>>::const_iterator it = qms.constBegin();
+	for (; it != qms.constEnd(); ++it)
+		res &= selectedModel->setItemData(selectedModel->index(i, it.key()), it.value());
+	return res;
 }
 
 
@@ -244,15 +273,15 @@ void GuiSelectionManager::addPB_clicked()
 	QModelIndex const idxToAdd = selIdx.first();
 	QModelIndex const idx = selectedLV->currentIndex();
 	int const srows = selectedModel->rowCount();
-	
+
 	QMap<int, QVariant> qm = availableModel->itemData(idxToAdd);
 	insertRowToSelected(srows, qm);
-	
+
 	selectionChanged(); //signal
 
 	if (idx.isValid())
 		selectedLV->setCurrentIndex(idx);
-	
+
 	updateHook();
 }
 
@@ -266,11 +295,11 @@ void GuiSelectionManager::deletePB_clicked()
 	QModelIndex idx = selIdx.first();
 	selectedModel->removeRow(idx.row());
 	selectionChanged(); //signal
-	
+
 	int nrows = selectedLV->model()->rowCount();
 	if (idx.row() == nrows) //was last item on list
 		idx = idx.sibling(idx.row() - 1, idx.column());
-	
+
 	if (nrows > 1)
 		selectedLV->setCurrentIndex(idx);
 	else if (nrows == 1)
@@ -291,11 +320,14 @@ void GuiSelectionManager::upPB_clicked()
 	int const pos = idx.row();
 	if (pos <= 0)
 		return;
-	
-	QMap<int, QVariant> qm = selectedModel->itemData(idx);
+
+	QMap<int, QMap<int, QVariant>> qms;
+	QList<QModelIndex>::const_iterator it = selIdx.constBegin();
+	for (; it != selIdx.constEnd(); ++it)
+		qms[it->column()] = selectedModel->itemData(*it);
 
 	selectedModel->removeRow(pos);
-	insertRowToSelected(pos - 1, qm);
+	insertRowToSelected(pos - 1, qms);
 
 	selectionChanged(); //signal
 
@@ -317,13 +349,16 @@ void GuiSelectionManager::downPB_clicked()
 	if (pos >= selectedModel->rowCount() - 1)
 		return;
 
-	QMap<int, QVariant> qm = selectedModel->itemData(idx);
+	QMap<int, QMap<int, QVariant>> qms;
+	QList<QModelIndex>::const_iterator it = selIdx.constBegin();
+	for (; it != selIdx.constEnd(); ++it)
+		qms[it->column()] = selectedModel->itemData(*it);
 
 	selectedModel->removeRow(pos);
-	insertRowToSelected(pos + 1, qm);
+	insertRowToSelected(pos + 1, qms);
 
 	selectionChanged(); //signal
-	
+
 	selectedLV->setCurrentIndex(idx.sibling(idx.row() + 1, idx.column()));
 	selectedHasFocus_ = true;
 	updateHook();
@@ -334,7 +369,7 @@ void GuiSelectionManager::availableLV_doubleClicked(const QModelIndex & idx)
 {
 	if (isSelected(idx) || !addPB->isEnabled())
 		return;
-	
+
 	if (idx.isValid())
 		selectedHasFocus_ = false;
 	addPB_clicked();
@@ -342,7 +377,7 @@ void GuiSelectionManager::availableLV_doubleClicked(const QModelIndex & idx)
 }
 
 
-bool GuiSelectionManager::eventFilter(QObject * obj, QEvent * event) 
+bool GuiSelectionManager::eventFilter(QObject * obj, QEvent * event)
 {
 	QEvent::Type etype = event->type();
 	if (obj == availableLV) {
@@ -355,27 +390,32 @@ bool GuiSelectionManager::eventFilter(QObject * obj, QEvent * event)
 			// This is designed to work both with the main enter key
 			// and the one on the numeric keypad.
 			if (keyPressed == Qt::Key_Enter || keyPressed == Qt::Key_Return) {
-				if (!keyModifiers)
-					addPB_clicked();
-				else if (keyModifiers == Qt::ControlModifier ||
-						keyModifiers == Qt::KeypadModifier  ||
-						keyModifiers == (Qt::ControlModifier | Qt::KeypadModifier)) {
+				if (!keyModifiers ||
+				    keyModifiers == Qt::ControlModifier ||
+				    keyModifiers == Qt::KeypadModifier  ||
+				    keyModifiers == (Qt::ControlModifier
+				                     | Qt::KeypadModifier)) {
 					if (addPB->isEnabled()) {
 						addPB_clicked();
-						okHook(); //signal
 					}
+					if (keyModifiers)
+						okHook(); //signal
 				}
 				event->accept();
 				return true;
 			}
+			else if (keyPressed == Qt::Key_Right) {
+				focusAndHighlight(selectedLV);
+				event->accept();
+				return true;
+			}
 		} else if (etype == QEvent::FocusIn) {
 			if (selectedHasFocus_) {
 				selectedHasFocus_ = false;
 				updateHook();
 			}
-			event->accept();
-			return true;
-		} 
+			return false;
+		}
 	} else if (obj == selectedLV) {
 		if (etype == QEvent::KeyPress) {
 			QKeyEvent * keyEvent = static_cast<QKeyEvent *>(event);
@@ -392,7 +432,7 @@ bool GuiSelectionManager::eventFilter(QObject * obj, QEvent * event)
 					updateHook();
 				} else
 					return QObject::eventFilter(obj, event);
-			} 
+			}
 			// Ctrl-Up activates upPB
 			else if (keyPressed == Qt::Key_Up) {
 				if (keyModifiers == Qt::ControlModifier) {
@@ -401,7 +441,7 @@ bool GuiSelectionManager::eventFilter(QObject * obj, QEvent * event)
 					event->accept();
 					return true;
 				}
-			} 
+			}
 			// Ctrl-Down activates downPB
 			else if (keyPressed == Qt::Key_Down) {
 				if (keyModifiers == Qt::ControlModifier) {
@@ -411,13 +451,17 @@ bool GuiSelectionManager::eventFilter(QObject * obj, QEvent * event)
 					return true;
 				}
 			}
+			else if (keyPressed == Qt::Key_Left) {
+				focusAndHighlight(availableLV);
+				event->accept();
+				return true;
+			}
 		} else if (etype == QEvent::FocusIn) {
 			if (!selectedHasFocus_) {
 				selectedHasFocus_ = true;
 				updateHook();
 			}
-			event->accept();
-			return true;
+			return false;
 		}
 	}
 	return QObject::eventFilter(obj, event);
