@@ -4,8 +4,8 @@
  * This file is part of LyX, the document processor.
  * Licence details can be found in the file COPYING.
  *
- * \author Jrgen Vigna
- * \author Lars Gullik Bjnnes
+ * \author Jürgen Vigna
+ * \author Lars Gullik Bjønnes
  *
  * Full author contact details are available in file CREDITS.
  */
@@ -25,21 +25,18 @@ namespace lyx {
 class InsetMarginal : public InsetFootlike {
 public:
 	///
-	InsetMarginal(Buffer const &);
+	InsetMarginal(Buffer *);
 	///
 	InsetCode lyxCode() const { return MARGIN_CODE; }
 	///
-	docstring name() const { return from_ascii("Marginal"); }
+	docstring layoutName() const { return from_ascii("Marginal"); }
 	///
-	int latex(odocstream &, OutputParams const &) const;
-	///
-	int plaintext(odocstream &, OutputParams const & runparams) const;
+	int plaintext(odocstringstream & ods, OutputParams const & op,
+	              size_t max_length = INT_MAX) const;
 	///
 	int docbook(odocstream &, OutputParams const & runparams) const;
-	///
-	docstring editMessage() const;
-	///
-	void addToToc(DocIterator const &);
+	/// Is the content of this inset part of the immediate (visible) text sequence?
+	bool isPartOfTextSequence() const { return false; }
 private:
 	///
 	Inset * clone() const { return new InsetMarginal(*this); }
